(function ($, Drupal, drupalSettings, klaro) {

  'use strict';

  /**
   * Initialize Klaro! with custom settings.
   */
  Drupal.behaviors.klaro = {
    attach: function (context, settings) {
      if (context !== document || !settings.klaro || !klaro) {
        return;
      }

      $(context).once('klaro').each(function () {
        // Create regular expressions from config.
        settings.klaro.config.apps.forEach(function (app, a_index) {
          app.cookies.forEach(function (cookie, c_index) {
            settings.klaro.config.apps[a_index]['cookies'][c_index] = [new RegExp(cookie[0], 'i'), cookie[1] || '/', cookie[2] || location.host];
            settings.klaro.config.apps[a_index].callback = Drupal.behaviors.klaro.attachBehaviors;
          });
        });

        if (settings.klaro.autoShow) {
          klaro.renderKlaro(settings.klaro.config);
        }
      });
    },

    /**
     * Attach Drupal behaviors from the just consented file(s).
     */
    attachBehaviors: function (consent, app) {
      if (!consent) {
        return;
      }
      if (app.onlyOnce && klaro.getManager(drupalSettings.klaro.config).executedOnce[app.name]) {
        return;
      }

      let knownBehaviors = Object.keys(Drupal.behaviors);
      $('[data-name="' + app.name + '"][src]').on('load', function () {
        for (let name in Drupal.behaviors) {
          if (knownBehaviors.indexOf(name) === -1) {
            // Execute newly added behaviors. This is somewhat guesswork, as
            // the order of execution is undetermined based on file load order.
            Drupal.behaviors[name].attach(document, drupalSettings);
            knownBehaviors.push(name);
          }
        }
      });
    },
  };

  /**
   * Open the Klaro! consent notice from links & buttons.
   */
  Drupal.behaviors.klaroLinkOpen = {
    attach: function (context, settings) {
      $('body', context).once('klaroLinkOpen').on('click', '[rel*="open-consent-manager"]', function (e, a, b) {
        klaro.renderKlaro(settings.klaro.config, true, true);
        e.preventDefault();
        return false;
      });
    }
  };

  /**
  * Deny all (except of required) consents by click on links & buttons.
  */
  Drupal.behaviors.klaroLinkDeclineAll = {
    attach: function (context, settings) {
      $('body', context).once('klaroLinkDeclineAll').on('click', '[rel*="consent-manager-decline-all"]', function (e, a, b) {
        var consent_manager = klaro.getManager(settings.klaro.config);
        consent_manager.changeAll(false);
        consent_manager.saveAndApplyConsents();
        e.preventDefault();
        return false;
      });
    }
  };

})(jQuery, Drupal, drupalSettings, klaro);
